#include <avr/io.h>
#include <avr/pgmspace.h>
#include <inttypes.h>
#include <stdio.h>

#include "rtc.h"
#include "scheduler.h"

// 3,6864MHz Frequenz
#define UART_BAUDRATE	115200UL

uint16_t taskId = 0;

/*
 * Wird einmal pro Sekunde ausgefhrt
 */
uint8_t wiederholFunktion(uint16_t parameter, date_t* taskTime) {
	if(parameter) {
		// Datum und Uhrzeit ausgeben
		uint8_t woDay = rtcDayOfWeek(&rtcTime);
		switch(woDay) {
			case RTC_MONDAY:		puts_P(PSTR("Mo, ")); break;
			case RTC_TUESDAY:		puts_P(PSTR("Di, ")); break;
			case RTC_WEDNESDAY:		puts_P(PSTR("Mi, ")); break;
			case RTC_THURSDAY:		puts_P(PSTR("Do, ")); break;
			case RTC_FRIDAY:		puts_P(PSTR("Fr, ")); break;
			case RTC_SATURDAY:		puts_P(PSTR("Sa, ")); break;
			case RTC_SUNDAY:		puts_P(PSTR("So, ")); break;
		}
		printf_P(PSTR("%.2d.%.2d.%.4d %.2d:%.2d.%.2d\r"), rtcTime.day+1,
														 rtcTime.month+1,
														 rtcTime.year,
														 rtcTime.hour,
														 rtcTime.min,
														 rtcTime.sec);
	} else {
		puts_P(PSTR("Parameter = 0\r"));
	}

	// Nchsten Ausfhrungszeitpunkt setzen
	rtcDateAdd(taskTime, RTC_SEC, 1);
		
	return SCHEDULER_REPEAT_TASK;
}

/*
 * Wird ein einziges Mal ausgefhrt.
 */
uint8_t einmalFunktion(uint16_t parameter, date_t* taskTime) {
	schedulerEditTaskParam(taskId, 1);
	puts_P(PSTR("Parameter = 1\r"));
	return SCHEDULER_TERMINATE_TASK;
}

int uartPut(char c) {
	while ( !( UCSR1A & (1<<UDRE)) );
	UDR1 = c;
	
	return 0;
}

int main(void) {
	
	rtcInit();
	schedulerInit();
	
	// UART konfigurieren
	UCSR1B = (1 << TXEN);		// Transmitter Enable
	UBRR1L = F_OSC/(16*UART_BAUDRATE)-1;
	
	fdevopen(uartPut, NULL, 0);
	
	
	// Aufgaben an den Scheduler bergeben
	
	date_t taskTime = schedulerTime;	// Wichtig: nicht rtcTime!
	taskId = schedulerAddTask(wiederholFunktion, &taskTime, 0);	// Parameter = 0
	
	rtcDateAdd(&taskTime, RTC_SEC, 10);
	schedulerAddTask(einmalFunktion, &taskTime, 0);
	
	// Main-Loop
	while(1) {
		schedulerCheck();
	}
}
