/*
 * Copyright:      Fabian Maximilian Thiele  mailto:me@apetech.de
 * Author:         Fabian Maximilian Thiele
 * Remarks:        this Copyright must be included
 * known Problems: none
 * Version:        1.0
 * Description:    Scheduler mit dynamischer Aufgaben-Liste
 * 
 */

#ifndef SCHEDULER_H
#define SCHEDULER_H

#include "rtc.h"


//
// Defines
//

#define SCHEDULER_TERMINATE_TASK	0
#define SCHEDULER_REPEAT_TASK		1


//
// Typedefs
//

/* 
 * SchedulerTask Calback Funktion, bekommt neben einem Benutzer-Definiertem
 * 16-Bit integer, auch die Ausfhrungszeit bergeben.
 * Soll der Task zu einem spteren Zeitpunkt erneut ausgefhrt werden, kann
 * der Zeitpunkt in das date_t struct geschrieben werden. Auerdem muss die
 * Funktion dann SCHEDULER_REPEAT_TASK zurckgeben. Gibt die Funktion statt
 * dessen SCHEDULER_TERMINATE_TASK zurck, wird der Task aus der Liste 
 * gelscht.
 */
typedef uint8_t (*schedulerCallback_t)(uint16_t, date_t*);

/*
 * Prototyp der Scheduler Listenelemente (Ein Eintrag beansprucht 17 Byte
 * Daten, auerdem bentigt die malloc()-Funktion 2 weitere Bytes.
 */
typedef struct {
	void* nextTaskPtr;
	uint16_t taskId;
	schedulerCallback_t task;
	uint16_t taskParam;
	date_t taskDate;
} schedulerTask_t;


//
// Funktions Prototypen
//

// Scheduler-Zeit initialisieren
void schedulerInit(void);
// Scheduler Liste checken (regelmig aus der Main-Loop aufrufen!)
void schedulerCheck(void);
// Fgt dem Scheduler einen Task hinzu. Es wird die TaskID zurckgegeben
uint16_t schedulerAddTask(schedulerCallback_t task, date_t* taskDate, uint16_t taskParam);
// ndert den Ausfhrungszeitpunkt eines Tasks
void schedulerEditTaskTime(uint16_t taskId, date_t* taskDate);
// ndert den bergabe-Paramter eines Tasks
void schedulerEditTaskParam(uint16_t taskId, uint16_t taskParam);
// Fhrt den spezifizierten Task aus, egal ob der Ausfhrungszeitpunkt erreicht
// ist oder nicht.
void schedulerForceTask(uint16_t taskId);
// Entfernt einen Task aus der Scheduler Liste
void schedulerRemoveTask(uint16_t taskId);


//
// Globale Variablen
//

// Scheduler Zeit
extern volatile date_t schedulerTime;
// Scheduler Eintrge
extern volatile uint8_t schedulerLength;

#endif
